from django.shortcuts import render, redirect
from django.contrib.auth import login
from django.contrib.auth.models import User
from django.contrib.auth import logout
from django.contrib import messages
from django.views.decorators.csrf import csrf_exempt
from django.http import JsonResponse
import csv
import os
import random
from datetime import datetime

# Fichier CSV pour stocker les scores
CSV_FILE = "scores.csv"
USER_FILE = "users.csv"
DIFFICULTY_FILE = "difficulties.csv"
DAILY_CHALLENGE_FILE = "daily_challenges.csv"

def init_csv():
    if not os.path.exists(CSV_FILE):
        with open(CSV_FILE, 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(["user_id", "score"])

def init_user_csv():
    if not os.path.exists(USER_FILE):
        with open(USER_FILE, 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(["user_id", "username"])

#################
# Page de Login #
#################

def login_view(request):
    # se log ou bien créer un compte si l'utilisateur n'existe pas sans mot de passe 
    if request.method == "POST":
        username = request.POST.get("username")
        user = User.objects.filter(username=username).first()
        if user is None:
            user = User.objects.create_user(username=username, password=None)
            user.save()
            add_user_to_csv(username, user.id)
        login(request, user)
        request.session['user_id'] = user.id
        return redirect("challenges")
    return render(request, "login.html")
        

def add_user_to_csv(username, django_user_id):
    """Ajoute un utilisateur avec un ID Django dans le fichier 'users.csv'."""
    user_id = django_user_id  # Utiliser l'ID Django
    with open(USER_FILE, 'a', newline='') as file:
        writer = csv.writer(file)
        writer.writerow([user_id, username])  # Ajouter l'ID Django et le nom d'utilisateur
    return user_id

##################
# Page de Logout #
##################

def logout_view(request):
    logout(request)
    return redirect('login')

##################
# Page de Result #
##################

def ranking_view(request):
    """Afficher le classement des scores avec les nom d'utilisateur."""
    # Lire les scores depuis le fichier CSV
    scores = read_scores()
    
    # Lire les utilisateurs depuis la base de données Django
    users = read_users()
    
    # Créer le classement en associant le score avec le nom d'utilisateur
    ranking = []
    for user_id, score in scores.items():
        if int(user_id) in users:  # Associer le user_id avec l'utilisateur dans le dictionnaire
            ranking.append({"username": users[int(user_id)], "score": score})
    
    # Trier le classement par score (du plus élevé au plus bas)
    ranking.sort(key=lambda x: x['score'], reverse=True)
    
    print(f"Classement: {ranking}")  # Debug : Afficher le classement
    
    # Passer le classement à la vue pour l'afficher
    return render(request, "ranking.html", {"ranking": ranking})

def read_users():
    """Lire les utilisateurs depuis django"""
    users = {}
    for user in User.objects.all():
        users[user.id] = user.username

    print(f"Utilisateurs: {users}")
    return users

#####################
# Page de Challenge #
#####################

def challenges_view(request):
    # Redirection vers la page de login en cas de session non authentifié
    if not request.user.is_authenticated:
        return redirect("login")
    
    user_id = request.session.get('user_id')  # Utiliser l'ID stocké en session
    today = datetime.today().strftime('%Y-%m-%d')  # Récupérer la date du jour

    # Vérifier si l'utilisateur a déjà généré ses défis pour aujourd'hui
    user_challenges = read_user_daily_challenges(user_id, today)

    if user_challenges:
        # Si les défis existent déjà pour aujourd'hui, on les récupère depuis le fichier
        selected_challenges = user_challenges
        print(f"Défis récupérés pour aujourd'hui: {len(selected_challenges)} défis")
    else:
        # Sinon, générer de nouveaux défis pour cet utilisateur
        print("Aucun défi généré pour aujourd'hui, génération de nouveaux défis.")
        selected_challenges = generate_daily_challenges(user_id)  # Générer et enregistrer de nouveaux défis

    # Rechercher la difficulté de l'user connecté
    user_difficulty = get_user_difficulty(user_id)
    print(f"Difficulté de l'utilisateur {user_id}: {user_difficulty}")

    selected_challenges = read_challenges_from_ids(selected_challenges)
    # Passer les défis du jour à la vue
    return render(request, "challenges.html", {"challenges": selected_challenges, "user_difficulty": user_difficulty})

def get_user_difficulty(user_id):
    """Récupérer la difficulté de l'utilisateur à partir du fichier 'difficulties.csv'."""
    difficulty = "moyen"  # Valeur par défaut
    if os.path.exists(DIFFICULTY_FILE):
        with open(DIFFICULTY_FILE, "r", newline="", encoding="utf-8") as file:
            reader = csv.reader(file)
            for row in reader:
                if len(row) >= 2 and row[0] == str(user_id):  # Vérifier l'ID utilisateur
                    difficulty = row[1]  # Récupérer la difficulté
                    break
    return difficulty

def read_user_daily_challenges(user_id, today):
    """Lire les défis du jour pour un utilisateur donné depuis le fichier daily_challenges.csv."""
    user_challenges = []
    if os.path.exists(DAILY_CHALLENGE_FILE):
        with open(DAILY_CHALLENGE_FILE, 'r', newline='', encoding='utf-8') as file:
            reader = csv.reader(file)
            for row in reader:
                if len(row) >= 3 and row[1] == str(user_id) and row[0] == today:
                    user_challenges = row[2:]  # Extraire les IDs des défis du jour
                    break
    return user_challenges

def generate_daily_challenges(user_id):
    """Génère et enregistre les défis du jour pour l'utilisateur."""
    challenges = read_challenges()
    user_difficulty = get_user_difficulty(user_id)
    user_difficulty = user_difficulty.capitalize()
    daily_challenges = []
    categories = ["Au bureau (poste)", "Au bureau (espaces communs)", "Extérieur"]
    random.shuffle(categories)  # Mélanger les catégories
    for category in categories:
        challenges_in_category = [c for c in challenges[user_difficulty] if c["lieu"] == category]
        if challenges_in_category:
            challenge = random.choice(challenges_in_category)
            daily_challenges.append(challenge["id"])

    save_daily_challenges_to_csv(user_id, daily_challenges)  # Enregistrer les défis du jour
    return daily_challenges

def save_daily_challenges_to_csv(user_id, challenge_ids):
    """Sauvegarder les défis du jour dans un fichier CSV"""
    today = datetime.today().strftime('%Y-%m-%d')
    with open(DAILY_CHALLENGE_FILE, 'a', newline='', encoding='utf-8') as file:
        writer = csv.writer(file)
        writer.writerow([today, user_id] + challenge_ids)

def update_score(user_id, points):
    """Mettre à jour le score d'un utilisateur."""
    scores = read_scores()
    if str(user_id) in scores:
        scores[str(user_id)] += points
    else:
        scores[str(user_id)] = points

    with open(CSV_FILE, 'w', newline='') as file:
        writer = csv.writer(file)
        writer.writerow(["user_id", "score"])  # En-tête
        for user_id, score in scores.items():
            writer.writerow([user_id, score])

def complete_challenge(request):
    if request.method == "POST":
        challenge_id = request.POST.get("challenge_id")
        user_id = request.session.get('user_id')  # Récupérer l'ID utilisateur depuis la session
        points = get_challenge_points(challenge_id)

        if points:
            update_score(user_id, points)  # Mettre à jour le score de l'utilisateur
            return JsonResponse({"status": "success", "points": points})

    return JsonResponse({"status": "error", "message": "Invalid request"}, status=400)

def read_challenges():
    """Lire tous les défis depuis un fichier CSV avec des colonnes séparées par des virgules."""
    challenges = {"Facile": [], "Moyen": [], "Difficile": []}
    if os.path.exists("challenges.csv"):
        with open("challenges.csv", 'r', newline='', encoding='utf-8') as file:
            reader = csv.reader(file)
            for row in reader:
                if len(row) >= 6:
                    challenge = {
                        "id": row[0],
                        "description": row[1],
                        "level": row[2],
                        "lieu": row[3],
                        "points": row[4],
                        "icon": row[5]
                    }
                    challenges[challenge["level"]].append(challenge)
    return challenges

def read_challenges_from_ids(challenge_ids):
    """Récupérer les défis à partir de leurs IDs."""
    challenges = read_challenges()
    selected_challenges = []
    for level in challenges:
        for challenge in challenges[level]:
            if challenge["id"] in challenge_ids:
                selected_challenges.append(challenge)
    return selected_challenges

def get_challenge_points(challenge_id):
    """Récupérer les points associés à un défi donné."""
    challenges = read_challenges()
    for level in challenges:
        for challenge in challenges[level]:
            if challenge["id"] == challenge_id:
                return int(challenge["points"])
    return None

@csrf_exempt
def update_difficulty(request):
    if request.method == "POST":
        user_id = request.session.get('user_id')
        difficulty = request.POST.get("difficulty")
        if difficulty in ["Facile", "Moyen", "Difficile"]:
            save_user_difficulty(user_id, difficulty)
            return JsonResponse({"status": "success"})
    return JsonResponse({"status": "error", "message": "Invalid request"}, status=400)

#####################
# Page de Mise à jour difficulty #
#####################

@csrf_exempt
def update_difficulty(request):
    if request.method == "POST":
        user_id = request.session.get('user_id')
        difficulty = request.POST.get("difficulty")
        if difficulty in ["facile", "moyen", "difficile"]:
            # Save user difficulty in file or database
            save_user_difficulty(user_id, difficulty)
            return JsonResponse({"status": "success"})
        else:
            return JsonResponse({"status": "error", "message": "Invalid difficulty"})
    return JsonResponse({"status": "error", "message": "Invalid request"}, status=400)

##########
# Commun #
##########

def read_scores():
    """Lire les scores depuis un fichier CSV."""
    scores = {}
    if os.path.exists(CSV_FILE):
        with open(CSV_FILE, 'r', newline='') as file:
            reader = csv.reader(file)
            next(reader)  # Ignorer l'en-tête
            for row in reader:
                if len(row) >= 2:
                    scores[row[0]] = int(row[1])  # Ajouter le score à un dictionnaire
    return scores

def save_user_difficulty(user_id, difficulty):
    """Remplace si l'user à déjà sauvegarder sa difficulté sinon sauvegarde la le tout dans un fichier CSV."""
    data = []
    if os.path.exists(DIFFICULTY_FILE):
        with open(DIFFICULTY_FILE, "r", newline="", encoding="utf-8") as file:
            reader = csv.reader(file)
            for row in reader:
                if len(row) >= 2 and row[0] == str(user_id):
                    data.append([user_id, difficulty])
                else:
                    data.append(row)

    with open(DIFFICULTY_FILE, "w", newline="", encoding="utf-8") as file:
        writer = csv.writer(file)
        for row in data:
            writer.writerow(row)
        if [user_id, difficulty] not in data:
            writer.writerow([user_id, difficulty])
